from naas_drivers.driver import OutDriver
from slack import WebClient
from slack.errors import SlackApiError
import json


class Slack(OutDriver):
    client = None

    def connect(self, key):
        self._key = key
        self.client = WebClient(token=self._key)
        self.connected = True
        return self

    def __upload_or_link(self, data):
        read_data = data
        if "." in data:
            try:
                read_data = self.__upload_file(data)
            except OSError:
                pass
        return self.__pureimg(read_data)

    def __pureimg(self, data1):
        data1 = '[{"text": "", "image_url": "' + data1 + '"}]'
        data1 = [json.loads(data1[1:-1])]
        return data1

    def __upload_file(self, path):
        response = self.client.files_upload(channel="#theta", file=path)
        return response["file"]["permalink"]

    def send(self, channel, text, image=None):
        self.check_connect()
        try:
            attachments = None
            if image:
                attachments = self.__upload_or_link(image)
            response = self.client.chat_postMessage(
                channel=channel, text=text, attachments=attachments
            )
            assert response["message"]["text"] == text
            print("Message send")
        except SlackApiError as e:
            # You will get a SlackApiError if "ok" is False
            assert e.response["ok"] is False
            assert e.response["error"]  # str like 'invalid_auth', 'channel_not_found'
            print(f"Got an error: {e.response['error']}")



from naas_drivers.driver import InDriver, OutDriver
import pandas as pd
import requests
from datetime import datetime
import os
import string


class HSCRUD:
    # class HSCRUD(CRUD):
    def __init__(self, base_url, req_headers, params):
        self.req_headers = req_headers
        self.params = params
        self.base_url = base_url
        self.model_name = self.base_url.split("/")[-1]

        # Manage message
        message_dict = {"contacts": "Contact", "company": "Company", "deals": "Deal"}
        self.msg = message_dict[self.model_name]

    def __values_format(self, data):
        for key, value in data["properties"].items():
            # Check if value is not None
            if value is not None:
                # Force format to string  & delete space
                value = str(value).strip()
                # Specific rules
                if key == "firstname":
                    value = value.replace("-", " ")
                    value = string.capwords(value).replace(" ", "-")
                elif key == "lastname":
                    value = value.upper()
                elif key == "phone":
                    value = value.replace(" ", "").replace(".", "")
                elif key == "closedate":
                    try:
                        value = datetime.strptime(value, "%d/%m/%Y")
                        value = str(int(value.timestamp())) + "000"
                    except ValueError:
                        print(
                            f"Close date '{value}' is in wrong format.\n"
                            "Please change it to %d/%m/%Y."
                        )
                elif key == "amount":
                    value = value.replace(".", "")
                # Change value in dict
                data["properties"][key] = value
        return data

    def __get_by_page(self, params):
        req = requests.get(
            url=f"{self.base_url}/",
            headers=self.req_headers,
            params=params,
            allow_redirects=False,
        )
        req.raise_for_status()
        return req.json()

    def get_all(self):
        items = []
        params = self.params
        more_page = True
        while more_page:
            data = self.__get_by_page(params)
            for row in data["results"]:
                properties = row["properties"]
                items.append(properties)
            if "paging" in data:
                params["after"] = data["paging"]["next"]["after"]
            else:
                more_page = False
        params.pop("after")
        self.params = params
        df = pd.DataFrame(items).reset_index(drop=True)
        return df

    def get(self, uid):
        try:
            req = requests.get(
                url=f"{self.base_url}/{uid}",
                headers=self.req_headers,
                params=self.params,
                allow_redirects=False,
            )
            req.raise_for_status()
            res = req.json()
            return res
        except requests.HTTPError as err:
            err_code = err.response.status_code
            if err_code == 404:
                print(f"{self.msg} id='{uid}' does not exist.")

    def patch(self, uid, data):
        data = self.__values_format(data)
        try:
            req = requests.patch(
                url=f"{self.base_url}/{uid}",
                headers=self.req_headers,
                params=self.params,
                json=data,
                allow_redirects=False,
            )
            req.raise_for_status()
            res = req.json()
            # Message success
            print(f"{self.msg} (id={uid}) successfully updated.")
            return res
        except requests.HTTPError as err:
            err_code = err.response.status_code
            err_msg = err.response.json()
            if err_code == 404:
                print(f"{self.msg} id='{uid}' does not exist.")
            else:
                print(f"{err_code}: {err_msg}")

    def send(self, data):
        data = self.__values_format(data)
        try:
            req = requests.post(
                url=f"{self.base_url}/",
                headers=self.req_headers,
                params=self.params,
                json=data,
                allow_redirects=False,
            )
            req.raise_for_status()
            res = req.json()
            uid = res["id"]
            # Message success
            print(f"{self.msg} (id={uid}) successfully created.")
            return uid
        except requests.HTTPError as err:
            err_code = err.response.status_code
            err_msg = err.response.json()
            print(f"{err_code}: {err_msg}")

    def delete(self, uid):
        res = self.get(uid)
        if res is not None:
            try:
                req = requests.delete(
                    url=f"{self.base_url}/{uid}",
                    headers=self.req_headers,
                    params=self.params,
                    allow_redirects=False,
                )
                req.raise_for_status()
                # Message success
                print(f"{self.msg} (id={uid}) deleted.")
            except requests.HTTPError as err:
                err_code = err.response.status_code
                err_msg = err.response.json()
                print(f"{err_code}: {err_msg}")


class Contact(HSCRUD):
    def update(
        self,
        uid,
        email=None,
        firstname=None,
        lastname=None,
        phone=None,
        jobtitle=None,
        website=None,
        company=None,
        hubspot_owner_id=None,
    ):
        data = {
            "properties": {
                "email": email,
                "firstname": firstname,
                "lastname": lastname,
                "phone": phone,
                "jobtitle": jobtitle,
                "website": website,
                "company": company,
                "hubspot_owner_id": hubspot_owner_id,
            }
        }
        res = self.patch(uid, data)
        return res

    def create(
        self,
        email,
        firstname=None,
        lastname=None,
        phone=None,
        jobtitle=None,
        website=None,
        company=None,
        hubspot_owner_id=None,
    ):
        data = {
            "properties": {
                "email": email,
                "firstname": firstname,
                "lastname": lastname,
                "phone": phone,
                "jobtitle": jobtitle,
                "website": website,
                "company": company,
                "hubspot_owner_id": hubspot_owner_id,
            }
        }
        res = self.send(data)
        return res


class Deal(HSCRUD):
    def update(
        self,
        uid,
        dealname=None,
        dealstage=None,
        closedate=None,
        amount=None,
        hubspot_owner_id=None,
    ):
        data = {
            "properties": {
                "dealstage": dealstage,
                "dealname": dealname,
                "closedate": closedate,
                "amount": amount,
                "hubspot_owner_id": hubspot_owner_id,
            }
        }
        res = self.patch(uid, data)
        return res

    def create(
        self, dealname, dealstage, closedate=None, amount=None, hubspot_owner_id=None
    ):
        data = {
            "properties": {
                "dealstage": dealstage,
                "dealname": dealname,
                "closedate": closedate,
                "amount": amount,
                "hubspot_owner_id": hubspot_owner_id,
            }
        }
        res = self.send(data)
        return res


class Pipeline:
    def __init__(self, base_url, req_headers, params):
        self.req_headers = req_headers
        self.params = params
        self.base_url = base_url
        self.model_name = self.base_url.split("/")[-1]

    def get_all_pipeline(self):
        req = requests.get(
            url=f"{self.base_url}/",
            headers=self.req_headers,
            params=self.params,
            allow_redirects=False,
        )
        req.raise_for_status()
        return req.json()


class Pipelines(Pipeline):
    def get_all(self):
        data = self.get_all_pipeline()
        df = pd.DataFrame.from_records(data["results"])
        df = df.drop(["stages"], axis=1)
        df = df.sort_values(by=["displayOrder"]).reset_index(drop=True)
        return df


class Dealstage(Pipeline):
    def get_all(self):
        data = self.get_all_pipeline()
        items = []
        for row in data["results"]:
            pipeline = row["label"]
            stages = row["stages"]
            id_pipeline = row["id"]
            for stage in stages:
                label = stage["label"]
                display_order = stage["displayOrder"]
                id_dealstage = stage["id"]
                created_at = stage["createdAt"]
                updated_at = stage["updatedAt"]
                archived = stage["archived"]

                deal_stage = {
                    "pipeline": pipeline,
                    "id_pipeline": id_pipeline,
                    "dealstage": label,
                    "id_dealstage": id_dealstage,
                    "displayOrder": display_order,
                    "createdAt": created_at,
                    "updatedAt": updated_at,
                    "archived": archived,
                }
                items.append(deal_stage)
        df = pd.DataFrame(items)
        df = df.sort_values(by=["pipeline", "displayOrder"])
        df = df.reset_index(drop=True)
        return df


class Association:
    def __init__(self, base_url, req_headers, params):
        self.req_headers = req_headers
        self.params = params
        self.base_url = base_url
        self.model_name = self.base_url.split("/")[-1]

    def get(self, object_name, object_id, associate):
        objects = ["deal"]
        object_check = True
        if object_name not in objects:
            object_check = False
            print(
                f"Object '{object_name}' does not exist.\n"
                f"Please chose one in following list: {objects}"
            )
        associates = ["contact"]
        associate_check = True
        if associate not in associates:
            associate_check = False
            print(
                f"Associate '{associate}' does not exist.\n"
                f"Please chose one in following list: {associates}"
            )
        if object_check and associate_check:
            try:
                req = requests.get(
                    url=f"{self.base_url}/{object_name}/{object_id}/"
                    f"associations/{associate}",
                    headers=self.req_headers,
                    params=self.params,
                    allow_redirects=False,
                )
                req.raise_for_status()
                data = req.json()
                df = pd.DataFrame.from_records(data["results"])
                if len(df) == 0:
                    print(f"Object id='{object_id}' does not have associates.")
                return df
            except requests.HTTPError as err:
                err_code = err.response.status_code
                err_msg = err.response.json()
                print(f"{err_code}: {err_msg}")

    def create(self, object_name, object_id, associate, id_associate):
        objects = ["deal"]
        object_check = True
        if object_name not in objects:
            object_check = False
            print(
                f"Object '{object_name}' does not exist.\n"
                f"Please chose one in following list: {objects}"
            )
        associates = ["contact"]
        associate_check = True
        if associate not in associates:
            associate_check = False
            print(
                f"Associate '{associate}' does not exist.\n"
                f"Please chose one in following list: {associates}"
            )
        if object_check and associate_check:
            try:
                req = requests.put(
                    url=f"{self.base_url}/{object_name}/{object_id}/associations/"
                    f"{associate}/{id_associate}/{object_name}_to_{associate}",
                    headers=self.req_headers,
                    params=self.params,
                    allow_redirects=False,
                )
                req.raise_for_status()
                print(
                    f"{object_name} '{object_id}' and {associate} "
                    f"'{id_associate}' successfully associated !"
                )
            except requests.HTTPError as err:
                err_code = err.response.status_code
                err_msg = err.response.json()
                print(f"{err_code}: {err_msg}")


class Hubspot(InDriver, OutDriver):

    base_url = os.environ.get("HUBSPOT_API_URL", "https://api.hubapi.com/crm/v3")
    api_token = None

    def connect(self, api_token):
        # Init Hubspot attribute
        self.token = api_token
        self.req_headers = {
            "accept": "application/json",
            "content-type": "application/json",
        }
        self.params = {"limit": "100", "archived": "false", "hapikey": api_token}

        # Init end point
        self.obj_url = f"{self.base_url}/objects"
        self.pip_url = f"{self.base_url}/pipelines"
        self.contacts = Contact(
            f"{self.obj_url}/contacts", self.req_headers, self.params
        )
        self.company = HSCRUD(f"{self.obj_url}/company", self.req_headers, self.params)
        self.deals = Deal(f"{self.obj_url}/deals", self.req_headers, self.params)
        self.pipelines = Pipelines(
            f"{self.pip_url}/deals", self.req_headers, self.params
        )
        self.dealstages = Dealstage(
            f"{self.pip_url}/deals", self.req_headers, self.params
        )
        self.associations = Association(
            f"{self.obj_url}", self.req_headers, self.params
        )

        # Set connexion to active
        self.connected = True
        return self



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="naas-drivers",
    version="0.45.0",
    author="Martin Donadieu",
    author_email="martindonadieu@gmail.com",
    license="BSD",
    description="Drivers made to easy connect to any services",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jupyter-naas/drivers",
    packages=find_packages(exclude=["tests"]),
    extras_require={
        "dev": [
            "pytest>=5,<7",
            "pytest-mock>=3,<4",
            "requests-mock>=1,<2",
            "twine>=3,<4",
            "flake8>=3,<4",
            "black",
            "commitizen>=2,<3",
            "pytest-cov>=2,<3",
        ]
    },
    install_requires=[
        "imap_tools>=0,<1",
        "slackclient>=2,<3",
        "pymsteams>=0,<1",
        "pdfkit>=0,<1",
        "markdown2>=2,<3",
        "newsapi-python>=0,<1",
        "airtable-python-wrapper>=0,<1",
        "notion>0,<1",
        "pyjwt>=1,<3",
        "tensorflow>=2,<3",
        "pysftp>=0,<1",
        "htmlbuilder>=0,<1",
        "vaderSentiment>=3,<4",
        "chardet>=3,<5",
        "Cython>=0,<1",
        "idna>=2,<4",
        "inflection>=0,<1",
        "joblib>=0,<2",
        "more-itertools>=8,<9",
        "numpy>=1,<2",
        "ipython>=7,<8",
        "pandas>=1,<2",
        "pandas-datareader>=0,<1",
        "patsy>=0,<1",
        "pmdarima>=1,<2",
        "python-dateutil>=2,<3",
        "python-dotenv>=0,<1",
        "pytz>=2020,<2022",
        "plotly>=4,<5",
        "kaleido",
        "Quandl>=3,<4",
        "requests>=2,<3",
        "scikit-learn>=0,<1",
        "scipy>=1,<2",
        "six>=1,<2",
        "statsmodels>=0,<1",
        "urllib3>=1,<2",
        "xlrd>=1,<3",
        "pymongo>=3,<4",
        "pysftp>=0,<1",
        "md2pdf>=0,<1",
        "sendgrid>=6,<7",
        "escapism>=1,<2",
        "openpyxl>=3,<4",
        "google-api-python-client>=1,<2",
        "google-auth-httplib2>=0,<1",
        "google-auth-oauthlib>=0,<1",
        "gspread>=3,<4",
        "oauth2client>=4,<5",
        "geopy>=1,<3",
        "GitPython>=3,<4",
        "cson>=0,<1",
        "opencv-python>=4,<5",
        "pytesseract>=0,<1",
    ],
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: BSD License",
        "Framework :: Jupyter",
        "Operating System :: OS Independent",
    ],
)



from naas_drivers.driver import InDriver
import requests
import urllib.parse
import traceback


class AwesomeNotebooks(InDriver):

    __branch = "master"
    __repo = "jupyter-naas/awesome-notebooks"
    __naas_dl = "https://app.naas.ai/user-redirect/naas/downloader?url="
    __api_url = "https://api.github.com/repos/{REPO}/git/trees/{BRANCH}?recursive=1"
    __base_url = "https://github.com/{REPO}/blob/{BRANCH}/"
    __TOC_LIST_PREFIX = "-"

    def __create_md(self, list_files, naas_open):
        """create markdown index of all notebook files in cwd and sub folders"""
        md_lines = []
        last_folder = None
        for file in list_files:
            folders = file.get("folders")
            level = folders.count("/") + 1
            indent = "  " * level
            if not last_folder or folders != last_folder:
                indent = "  " * (level - 1)
                folder = folders.split("/")[-1]
                last_folder = folders
                md_lines.append(f"{indent} {self.__TOC_LIST_PREFIX} **{folder}**\n")
            url_path = file.get("url")
            md_filename = file.get("filename")
            indent = "  " * level
            open_url = url_path
            if naas_open:
                open_url = self.__naas_dl + url_path
            md_lines.append(
                f"{indent} {self.__TOC_LIST_PREFIX} [{md_filename}]({open_url})\n"
            )

        return "".join(md_lines)

    def __get_file_list(self):
        url = self.__api_url.replace("{REPO}", self.__repo).replace(
            "{BRANCH}", self.__branch
        )
        files_list = []
        try:
            r = requests.get(url)
            data = r.json()
            for ff in data.get("tree"):
                path = ff.get("path")
                if (
                    not path.startswith(".")
                    and path.endswith(".ipynb")
                    and not path.endswith("generatereadme.ipynb")
                ):
                    base = self.__base_url.replace("{REPO}", self.__repo).replace(
                        "{BRANCH}", self.__branch
                    )
                    good_url = f"{base}{urllib.parse.quote(path)}"
                    folders = path.split("/")
                    filename = folders.pop()
                    if len(folders) > 0:
                        main_name = folders[0]
                        folders = "/".join(folders)
                        folders = folders.replace("_", " ")
                        filename = filename.replace(f"{main_name}", "")
                        filename = filename.replace("_", " ")
                        filename = filename.replace(".ipynb", "")
                        filename = filename.strip()
                        files_list.append(
                            {"folders": folders, "filename": filename, "url": good_url}
                        )
        except Exception as e:
            print("__get_onboarding_list", e)
            traceback.print_exc()
        return files_list

    def connect(self, repo):
        self.__repo = repo if repo else self.__repo
        self.connected = True
        return self

    def get(self, md=True, open_in_naas=True):
        self.check_connect()
        if not md:
            return self.__get_file_list()
        else:
            arr = self.__get_file_list()
            text_md = self.__create_md(arr, open_in_naas)
            return text_md
